@@ -1,4 +1,4 @@
-/* $Id: airport_gui.cpp 26025 2013-11-17 13:53:33Z rubidium $ */
+/* $Id: airport_gui.cpp 27163 2015-02-22 15:26:27Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -27,9 +27,12 @@
 #include "core/geometry_func.hpp"
 #include "hotkeys.h"
 #include "vehicle_func.h"
+#include "gui.h"
 
 #include "widgets/airport_widget.h"
 
+#include "safeguards.h"
+
 
 static AirportClassID _selected_airport_class; ///< the currently visible airport class
 static int _selected_airport_index;            ///< the index of the selected airport in the current class or -1
@@ -43,7 +46,7 @@ void CcBuildAirport(const CommandCost &result, TileIndex tile, uint32 p1, uint32
 {
 	if (result.Failed()) return;
 
-	if (_settings_client.sound.confirm) SndPlayTileFx(SND_1F_SPLAT, tile);
+	if (_settings_client.sound.confirm) SndPlayTileFx(SND_1F_SPLAT_OTHER, tile);
 	if (!_settings_client.gui.persistent_buildingtools) ResetObjectToPlace();
 }
 
@@ -67,9 +70,9 @@ static void PlaceAirport(TileIndex tile)
 struct BuildAirToolbarWindow : Window {
 	int last_user_action; // Last started user action.
 
-	BuildAirToolbarWindow(const WindowDesc *desc, WindowNumber window_number) : Window()
+	BuildAirToolbarWindow(WindowDesc *desc, WindowNumber window_number) : Window(desc)
 	{
-		this->InitNested(desc, window_number);
+		this->InitNested(window_number);
 		if (_settings_client.gui.link_terraform_toolbar) ShowTerraformToolbar(this);
 		this->last_user_action = WIDGET_LIST_END;
 	}
@@ -79,6 +82,18 @@ struct BuildAirToolbarWindow : Window {
 		if (_settings_client.gui.link_terraform_toolbar) DeleteWindowById(WC_SCEN_LAND_GEN, 0, false);
 	}
 
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
+	{
+		if (!gui_scope) return;
+
+		if (!CanBuildVehicleInfrastructure(VEH_AIRCRAFT)) delete this;
+	}
+
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
@@ -99,14 +114,6 @@ struct BuildAirToolbarWindow : Window {
 	}
 
 
-	virtual EventState OnKeyPress(WChar key, uint16 keycode)
-	{
-		int num = CheckHotkeyMatch(airtoolbar_hotkeys, keycode, this);
-		if (num == -1) return ES_NOT_HANDLED;
-		this->OnClick(Point(), num, 1);
-		return ES_HANDLED;
-	}
-
 	virtual void OnPlaceObject(Point pt, TileIndex tile)
 	{
 		switch (this->last_user_action) {
@@ -142,15 +149,28 @@ struct BuildAirToolbarWindow : Window {
 		DeleteWindowById(WC_SELECT_STATION, 0);
 	}
 
-	static Hotkey<BuildAirToolbarWindow> airtoolbar_hotkeys[];
+	static HotkeyList hotkeys;
 };
 
-Hotkey<BuildAirToolbarWindow> BuildAirToolbarWindow::airtoolbar_hotkeys[] = {
-	Hotkey<BuildAirToolbarWindow>('1', "airport", WID_AT_AIRPORT),
-	Hotkey<BuildAirToolbarWindow>('2', "demolish", WID_AT_DEMOLISH),
-	HOTKEY_LIST_END(BuildAirToolbarWindow)
+/**
+ * Handler for global hotkeys of the BuildAirToolbarWindow.
+ * @param hotkey Hotkey
+ * @return ES_HANDLED if hotkey was accepted.
+ */
+static EventState AirportToolbarGlobalHotkeys(int hotkey)
+{
+	if (_game_mode != GM_NORMAL || !CanBuildVehicleInfrastructure(VEH_AIRCRAFT)) return ES_NOT_HANDLED;
+	Window *w = ShowBuildAirToolbar();
+	if (w == NULL) return ES_NOT_HANDLED;
+	return w->OnHotkey(hotkey);
+}
+
+static Hotkey airtoolbar_hotkeys[] = {
+	Hotkey('1', "airport", WID_AT_AIRPORT),
+	Hotkey('2', "demolish", WID_AT_DEMOLISH),
+	HOTKEY_LIST_END
 };
-Hotkey<BuildAirToolbarWindow> *_airtoolbar_hotkeys = BuildAirToolbarWindow::airtoolbar_hotkeys;
+HotkeyList BuildAirToolbarWindow::hotkeys("airtoolbar", airtoolbar_hotkeys, AirportToolbarGlobalHotkeys);
 
 static const NWidgetPart _nested_air_toolbar_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
@@ -165,11 +185,12 @@ static const NWidgetPart _nested_air_toolbar_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _air_toolbar_desc(
-	WDP_ALIGN_TOOLBAR, 0, 0,
+static WindowDesc _air_toolbar_desc(
+	WDP_ALIGN_TOOLBAR, "toolbar_air", 0, 0,
 	WC_BUILD_TOOLBAR, WC_NONE,
 	WDF_CONSTRUCTION,
-	_nested_air_toolbar_widgets, lengthof(_nested_air_toolbar_widgets)
+	_nested_air_toolbar_widgets, lengthof(_nested_air_toolbar_widgets),
+	&BuildAirToolbarWindow::hotkeys
 );
 
 /**
@@ -187,16 +208,6 @@ Window *ShowBuildAirToolbar()
 	return AllocateWindowDescFront<BuildAirToolbarWindow>(&_air_toolbar_desc, TRANSPORT_AIR);
 }
 
-EventState AirportToolbarGlobalHotkeys(WChar key, uint16 keycode)
-{
-	if (!CanBuildVehicleInfrastructure(VEH_AIRCRAFT)) return ES_NOT_HANDLED;
-	int num = CheckHotkeyMatch<BuildAirToolbarWindow>(_airtoolbar_hotkeys, keycode, NULL, true);
-	if (num == -1) return ES_NOT_HANDLED;
-	Window *w = ShowBuildAirToolbar();
-	if (w == NULL) return ES_NOT_HANDLED;
-	return w->OnKeyPress(key, keycode);
-}
-
 class BuildAirportWindow : public PickerWindowBase {
 	SpriteID preview_sprite; ///< Cached airport preview sprite.
 	int line_height;
@@ -208,29 +219,48 @@ class BuildAirportWindow : public PickerWindowBase {
 		DropDownList *list = new DropDownList();
 
 		for (uint i = 0; i < AirportClass::GetClassCount(); i++) {
-			list->push_back(new DropDownListStringItem(AirportClass::Get((AirportClassID)i)->name, i, false));
+			*list->Append() = new DropDownListStringItem(AirportClass::Get((AirportClassID)i)->name, i, false);
 		}
 
 		return list;
 	}
 
 public:
-	BuildAirportWindow(const WindowDesc *desc, Window *parent) : PickerWindowBase(parent)
+	BuildAirportWindow(WindowDesc *desc, Window *parent) : PickerWindowBase(desc, parent)
 	{
-		this->CreateNestedTree(desc);
+		this->CreateNestedTree();
 
 		this->vscroll = this->GetScrollbar(WID_AP_SCROLLBAR);
 		this->vscroll->SetCapacity(5);
 		this->vscroll->SetPosition(0);
 
-		this->FinishInitNested(desc, TRANSPORT_AIR);
+		this->FinishInitNested(TRANSPORT_AIR);
 
 		this->SetWidgetLoweredState(WID_AP_BTN_DONTHILIGHT, !_settings_client.gui.station_show_coverage);
 		this->SetWidgetLoweredState(WID_AP_BTN_DOHILIGHT, _settings_client.gui.station_show_coverage);
 		this->OnInvalidateData();
 
-		this->vscroll->SetCount(AirportClass::Get(_selected_airport_class)->GetSpecCount());
-		this->SelectFirstAvailableAirport(true);
+		/* Ensure airport class is valid (changing NewGRFs). */
+		_selected_airport_class = Clamp(_selected_airport_class, APC_BEGIN, (AirportClassID)(AirportClass::GetClassCount() - 1));
+		const AirportClass *ac = AirportClass::Get(_selected_airport_class);
+		this->vscroll->SetCount(ac->GetSpecCount());
+
+		/* Ensure the airport index is valid for this class (changing NewGRFs). */
+		_selected_airport_index = Clamp(_selected_airport_index, -1, ac->GetSpecCount() - 1);
+
+		/* Only when no valid airport was selected, we want to select the first airport. */
+		bool selectFirstAirport = true;
+		if (_selected_airport_index != -1) {
+			const AirportSpec *as = ac->GetSpec(_selected_airport_index);
+			if (as->IsAvailable()) {
+				/* Ensure the airport layout is valid. */
+				_selected_airport_layout = Clamp(_selected_airport_layout, 0, as->num_table - 1);
+				selectFirstAirport = false;
+				this->UpdateSelectSize();
+			}
+		}
+
+		if (selectFirstAirport) this->SelectFirstAvailableAirport(true);
 	}
 
 	virtual ~BuildAirportWindow()
@@ -287,7 +317,7 @@ public:
 				}
 
 				this->line_height = FONT_HEIGHT_NORMAL + WD_MATRIX_TOP + WD_MATRIX_BOTTOM;
-				size->height = this->vscroll->GetCapacity() * this->line_height;
+				size->height = 5 * this->line_height;
 				break;
 			}
 
@@ -395,7 +425,7 @@ public:
 		 * Never make the window smaller to avoid oscillating if the size change affects the acceptance.
 		 * (This is the case, if making the window bigger moves the mouse into the window.) */
 		if (top > bottom) {
-			ResizeWindow(this, 0, top - bottom);
+			ResizeWindow(this, 0, top - bottom, false);
 		}
 	}
 
@@ -527,8 +557,9 @@ static const NWidgetPart _nested_build_airport_widgets[] = {
 	NWidget(WWT_PANEL, COLOUR_DARK_GREEN), SetFill(1, 0), SetPIP(2, 0, 2),
 		NWidget(WWT_LABEL, COLOUR_DARK_GREEN), SetDataTip(STR_STATION_BUILD_AIRPORT_CLASS_LABEL, STR_NULL), SetFill(1, 0),
 		NWidget(WWT_DROPDOWN, COLOUR_GREY, WID_AP_CLASS_DROPDOWN), SetFill(1, 0), SetDataTip(STR_BLACK_STRING, STR_STATION_BUILD_AIRPORT_TOOLTIP),
+		NWidget(WWT_EMPTY, COLOUR_DARK_GREEN, WID_AP_AIRPORT_SPRITE), SetFill(1, 0),
 		NWidget(NWID_HORIZONTAL),
-			NWidget(WWT_MATRIX, COLOUR_GREY, WID_AP_AIRPORT_LIST), SetFill(1, 0), SetDataTip(0x501, STR_STATION_BUILD_AIRPORT_TOOLTIP), SetScrollbar(WID_AP_SCROLLBAR),
+			NWidget(WWT_MATRIX, COLOUR_GREY, WID_AP_AIRPORT_LIST), SetFill(1, 0), SetMatrixDataTip(1, 5, STR_STATION_BUILD_AIRPORT_TOOLTIP), SetScrollbar(WID_AP_SCROLLBAR),
 			NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_AP_SCROLLBAR),
 		EndContainer(),
 		NWidget(NWID_HORIZONTAL),
@@ -536,7 +567,6 @@ static const NWidgetPart _nested_build_airport_widgets[] = {
 			NWidget(WWT_LABEL, COLOUR_GREY, WID_AP_LAYOUT_NUM), SetResize(1, 0), SetFill(1, 0), SetDataTip(STR_BLACK_STRING, STR_NULL),
 			NWidget(WWT_PUSHARROWBTN, COLOUR_GREY, WID_AP_LAYOUT_INCREASE), SetMinimalSize(12, 0), SetDataTip(AWV_INCREASE, STR_NULL),
 		EndContainer(),
-		NWidget(WWT_EMPTY, COLOUR_DARK_GREEN, WID_AP_AIRPORT_SPRITE), SetFill(1, 0),
 		NWidget(WWT_EMPTY, COLOUR_DARK_GREEN, WID_AP_EXTRA_TEXT), SetFill(1, 0), SetMinimalSize(150, 0),
 	EndContainer(),
 	/* Bottom panel. */
@@ -556,8 +586,8 @@ static const NWidgetPart _nested_build_airport_widgets[] = {
 	EndContainer(),
 };
 
-static const WindowDesc _build_airport_desc(
-	WDP_AUTO, 0, 0,
+static WindowDesc _build_airport_desc(
+	WDP_AUTO, "build_station_air", 0, 0,
 	WC_BUILD_STATION, WC_BUILD_TOOLBAR,
 	WDF_CONSTRUCTION,
 	_nested_build_airport_widgets, lengthof(_nested_build_airport_widgets)
