@@ -1,4 +1,4 @@
-/* $Id: dedicated.cpp 20194 2010-07-19 17:46:53Z rubidium $ */
+/* $Id: dedicated.cpp 26482 2014-04-23 20:13:33Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,12 +13,15 @@
 
 #ifdef ENABLE_NETWORK
 
-char *_log_file; ///< File to reroute output of a forked OpenTTD to
+char *_log_file = NULL; ///< File to reroute output of a forked OpenTTD to
+FILE *_log_fd   = NULL; ///< File to reroute output of a forked OpenTTD to
 
 #if defined(UNIX) && !defined(__MORPHOS__)
 
 #include <unistd.h>
 
+#include "safeguards.h"
+
 #if (defined(SUNOS) && !defined(_LP64) && !defined(_I32LPx)) || defined(__HAIKU__)
 /* Solaris has, in certain situation, pid_t defined as long, while in other
  *  cases it has it defined as int... this handles all cases nicely.
@@ -39,20 +42,18 @@ void DedicatedFork()
 			exit(1);
 
 		case 0: { // We're the child
-			FILE *f;
-
 			/* Open the log-file to log all stuff too */
-			f = fopen(_log_file, "a");
-			if (f == NULL) {
+			_log_fd = fopen(_log_file, "a");
+			if (_log_fd == NULL) {
 				perror("Unable to open logfile");
 				exit(1);
 			}
 			/* Redirect stdout and stderr to log-file */
-			if (dup2(fileno(f), fileno(stdout)) == -1) {
+			if (dup2(fileno(_log_fd), fileno(stdout)) == -1) {
 				perror("Rerouting stdout");
 				exit(1);
 			}
-			if (dup2(fileno(f), fileno(stderr)) == -1) {
+			if (dup2(fileno(_log_fd), fileno(stderr)) == -1) {
 				perror("Rerouting stderr");
 				exit(1);
 			}
