@@ -1,4 +1,4 @@
-/* $Id: engine.cpp 25978 2013-11-13 21:21:47Z rubidium $ */
+/* $Id: engine.cpp 27607 2016-06-30 18:36:01Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -35,6 +35,8 @@
 #include "table/strings.h"
 #include "table/engines.h"
 
+#include "safeguards.h"
+
 EnginePool _engine_pool("Engine");
 INSTANTIATE_POOL_METHODS(Engine)
 
@@ -83,6 +85,7 @@ Engine::Engine(VehicleType type, EngineID base)
 	this->type = type;
 	this->grf_prop.local_id = base;
 	this->list_position = base;
+	this->preview_company = INVALID_COMPANY;
 
 	/* Check if this base engine is within the original engine data range */
 	if (base >= _engine_counts[type]) {
@@ -653,6 +656,7 @@ void StartupOneEngine(Engine *e, Date aging_date)
 	e->age = 0;
 	e->flags = 0;
 	e->company_avail = 0;
+	e->company_hidden = 0;
 
 	/* Don't randomise the start-date in the first two years after gamestart to ensure availability
 	 * of engines in early starting games.
@@ -851,6 +855,41 @@ void EnginesDailyLoop()
 	}
 }
 
+/**
+ * Clear the 'hidden' flag for all engines of a new company.
+ * @param cid Company being created.
+ */
+void ClearEnginesHiddenFlagOfCompany(CompanyID cid)
+{
+	Engine *e;
+	FOR_ALL_ENGINES(e) {
+		SB(e->company_hidden, cid, 1, 0);
+	}
+}
+
+/**
+ * Set the visibility of an engine.
+ * @param tile Unused.
+ * @param flags Operation to perform.
+ * @param p1 Unused.
+ * @param p2 Bit 31: 0=visible, 1=hidden, other bits for the #EngineID.
+ * @param text Unused.
+ * @return The cost of this operation or an error.
+ */
+CommandCost CmdSetVehicleVisibility(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
+{
+	Engine *e = Engine::GetIfValid(GB(p2, 0, 31));
+	if (e == NULL || _current_company >= MAX_COMPANIES) return CMD_ERROR;
+	if (!IsEngineBuildable(e->index, e->type, _current_company)) return CMD_ERROR;
+
+	if ((flags & DC_EXEC) != 0) {
+		SB(e->company_hidden, _current_company, 1, GB(p2, 31, 1));
+		AddRemoveEngineFromAutoreplaceAndBuildWindows(e->type);
+	}
+
+	return CommandCost();
+}
+
 /**
  * Accept an engine prototype. XXX - it is possible that the top-company
  * changes while you are waiting to accept the offer? Then it becomes invalid
@@ -864,7 +903,7 @@ void EnginesDailyLoop()
 CommandCost CmdWantEnginePreview(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	Engine *e = Engine::GetIfValid(p1);
-	if (e == NULL || e->preview_company != _current_company) return CMD_ERROR;
+	if (e == NULL || !(e->flags & ENGINE_EXCLUSIVE_PREVIEW) || e->preview_company != _current_company) return CMD_ERROR;
 
 	if (flags & DC_EXEC) AcceptEnginePreview(p1, _current_company);
 
@@ -1025,7 +1064,7 @@ CommandCost CmdRenameEngine(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 		if (reset) {
 			e->name = NULL;
 		} else {
-			e->name = strdup(text);
+			e->name = stredup(text);
 		}
 
 		MarkWholeScreenDirty();
