@@ -1,4 +1,4 @@
-/* $Id: array.hpp 18826 2010-01-16 14:22:19Z frosch $ */
+/* $Id: array.hpp 27363 2015-08-08 13:19:38Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -9,14 +9,16 @@
 
 /** @file array.hpp Array without an explicit maximum size. */
 
-#ifndef  ARRAY_HPP
-#define  ARRAY_HPP
+#ifndef ARRAY_HPP
+#define ARRAY_HPP
 
 #include "fixedsizearray.hpp"
 #include "str.hpp"
 
-/** Flexible array with size limit. Implemented as fixed size
- *  array of fixed size arrays */
+/**
+ * Flexible array with size limit. Implemented as fixed size
+ *  array of fixed size arrays
+ */
 template <class T, uint B = 1024, uint N = B>
 class SmallArray {
 protected:
@@ -28,11 +30,11 @@ protected:
 	SuperArray data; ///< array of arrays of items
 
 	/** return first sub-array with free space for new item */
-	FORCEINLINE SubArray& FirstFreeSubArray()
+	inline SubArray& FirstFreeSubArray()
 	{
 		uint super_size = data.Length();
 		if (super_size > 0) {
-			SubArray& s = data[super_size - 1];
+			SubArray &s = data[super_size - 1];
 			if (!s.IsFull()) return s;
 		}
 		return *data.AppendC();
@@ -40,11 +42,18 @@ protected:
 
 public:
 	/** implicit constructor */
-	FORCEINLINE SmallArray() { }
+	inline SmallArray()
+	{
+	}
+
 	/** Clear (destroy) all items */
-	FORCEINLINE void Clear() {data.Clear();}
+	inline void Clear()
+	{
+		data.Clear();
+	}
+
 	/** Return actual number of items */
-	FORCEINLINE uint Length() const
+	inline uint Length() const
 	{
 		uint super_size = data.Length();
 		if (super_size == 0) return 0;
@@ -52,28 +61,48 @@ public:
 		return (super_size - 1) * B + sub_size;
 	}
 	/** return true if array is empty */
-	FORCEINLINE bool IsEmpty() { return data.IsEmpty(); }
-	/** return true if array is full */
-	FORCEINLINE bool IsFull() { return data.IsFull() && data[N - 1].IsFull(); }
-	/** allocate but not construct new item */
-	FORCEINLINE T *Append() { return FirstFreeSubArray().Append(); }
-	/** allocate and construct new item */
-	FORCEINLINE T *AppendC() { return FirstFreeSubArray().AppendC(); }
-	/** indexed access (non-const) */
-	FORCEINLINE T& operator [] (uint index)
+	inline bool IsEmpty()
 	{
-		const SubArray& s = data[index / B];
-		T& item = s[index % B];
+		return data.IsEmpty();
+	}
+
+	/** return true if array is full */
+	inline bool IsFull()
+	{
+		return data.IsFull() && data[N - 1].IsFull();
+	}
+
+	/** allocate but not construct new item */
+	inline T *Append()
+	{
+		return FirstFreeSubArray().Append();
+	}
+
+	/** allocate and construct new item */
+	inline T *AppendC()
+	{
+		return FirstFreeSubArray().AppendC();
+	}
+
+	/** indexed access (non-const) */
+	inline T& operator[](uint index)
+	{
+		const SubArray &s = data[index / B];
+		T &item = s[index % B];
 		return item;
 	}
 	/** indexed access (const) */
-	FORCEINLINE const T& operator [] (uint index) const
+	inline const T& operator[](uint index) const
 	{
-		const SubArray& s = data[index / B];
-		const T& item = s[index % B];
+		const SubArray &s = data[index / B];
+		const T &item = s[index % B];
 		return item;
 	}
 
+	/**
+	 * Helper for creating a human readable output of this data.
+	 * @param dmp The location to dump to.
+	 */
 	template <typename D> void Dump(D &dmp) const
 	{
 		dmp.WriteLine("capacity = %d", Tcapacity);
@@ -81,7 +110,7 @@ public:
 		dmp.WriteLine("num_items = %d", num_items);
 		CStrA name;
 		for (uint i = 0; i < num_items; i++) {
-			const T& item = (*this)[i];
+			const T &item = (*this)[i];
 			name.Format("item[%d]", i);
 			dmp.WriteStructT(name.Data(), &item);
 		}
