@@ -1,4 +1,4 @@
-/* $Id: network_gui.cpp 27431 2015-11-01 11:59:17Z frosch $ */
+/* $Id: network_gui.cpp 27591 2016-05-29 19:05:11Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -1892,7 +1892,7 @@ struct NetworkClientListWindow : Window {
 		int diff = (num + WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM) - (this->GetWidget<NWidgetBase>(WID_CL_PANEL)->current_y);
 		/* If height is changed */
 		if (diff != 0) {
-			ResizeWindow(this, 0, diff);
+			ResizeWindow(this, 0, diff, false);
 			return false;
 		}
 		return true;
