@@ -1,4 +1,4 @@
-/* $Id: newgrf_house.cpp 26580 2014-05-11 18:02:11Z rubidium $ */
+/* $Id: newgrf_house.cpp 27590 2016-05-29 19:01:50Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -399,7 +399,8 @@ static uint32 GetDistanceFromNearbyHouse(uint8 parameter, TileIndex tile, HouseI
 		case 0x66: {
 			TileIndex testtile = GetNearbyTile(parameter, this->tile);
 			if (!IsTileType(testtile, MP_HOUSE)) return 0xFFFFFFFF;
-			HouseSpec *hs = HouseSpec::Get(GetHouseType(testtile));
+			HouseID nearby_house_id = GetHouseType(testtile);
+			HouseSpec *hs = HouseSpec::Get(nearby_house_id);
 			/* Information about the grf local classid if the house has a class */
 			uint houseclass = 0;
 			if (hs->class_id != HOUSE_NO_CLASS) {
@@ -408,8 +409,8 @@ static uint32 GetDistanceFromNearbyHouse(uint8 parameter, TileIndex tile, HouseI
 			}
 			/* old house type or grf-local houseid */
 			uint local_houseid = 0;
-			if (this->house_id < NEW_HOUSE_OFFSET) {
-				local_houseid = this->house_id;
+			if (nearby_house_id < NEW_HOUSE_OFFSET) {
+				local_houseid = nearby_house_id;
 			} else {
 				local_houseid = (hs->grf_prop.grffile == this->ro.grffile ? 1 : 2) << 8;
 				local_houseid |= hs->grf_prop.local_id;
